@@ -15,6 +15,11 @@ FLOAT_TYPE = "flt"
 
 PLTFRM = sys.platform
 
+SYSTEM_ENTRIES = {
+    "help": ("?help?", "?h?"),
+    "cancel": ("?cancel?", "cancel", "", "cancel.")
+}
+
 
 def typing_print(message: str, *args, **kwargs):
     """
@@ -33,29 +38,35 @@ def typing_print(message: str, *args, **kwargs):
     print("\n", end="", flush=True, *args, **kwargs)
 
 
-def get_user_input(message: str, expected_type: str, can_cancel: bool = True, print_func=print, allow_newlines: bool = True, *args, **kwargs):
+def get_user_input(msg: str, expected_type: str, can_cancel: bool = True, print_func=print, allow_newlines: bool = True, help_msg: str = None *args, **kwargs):
     """
     Description:
         get user input and returns the expected datatype
-    :param message: the message to tell the user what to type
+    :param msg: the message to tell the user what to type
     :param expected_type: a string consisting of the type of data the user needs to enter
     :param can_cancel: a bool that decides if the user can cancel
     :param print_func: the function to which the messages be printed
+    :param allow_newlines: weather or not the user can enter emulated newlines and allow the system to parse them
+    :param help_msg: the message to print if the user enters the string '?help?'
     :return: The data in the type equal to expected_type or None if the user cancels
     """
     print_func(message, *args, **kwargs)
     data = input(">>> ")
-    if data.lower() in ("cancel", "", "cancel."):
+    if data.lower() in SYSTEM_ENTRIES["cancel"]:
         if can_cancel:
             return None
         print_func("invalid entry. try again")
-        return get_user_input(message, expected_type, can_cancel, print_func)
+        return get_user_input(msg, expected_type, can_cancel=can_cancel, print_func=print_func, allow_newlines=allow_newlines, help_msg=help_msg)
+    if data.lower() in SYSTEM_ENTRIES["help"]:
+        if help_msg is not None:
+            print(help_msg)
+            return get_user_input(msg, expected_type, can_cancel=can_cancel, print_func=print_func, allow_newlines=allow_newlines, help_msg=help_msg)
     if expected_type == INT_TYPE:
         try:
             data = int(data)
         except ValueError:
             print_func("invalid entry. try again")
-            return get_user_input(message, expected_type, can_cancel, print_func)
+            return get_user_input(msg, expected_type, can_cancel=can_cancel, print_func=print_func, allow_newlines=allow_newlines, help_msg=help_msg)
         return data
     elif expected_type == STR_TYPE:
         if allow_newlines:
@@ -67,13 +78,13 @@ def get_user_input(message: str, expected_type: str, can_cancel: bool = True, pr
         if data.lower() in ("y", "yes", "yeah", "1", "true", "yup"):
             return True
         print_func("invalid entry. try again.")
-        return get_user_input(message, expected_type, can_cancel, print_func)
+        return get_user_input(msg, expected_type, can_cancel=can_cancel, print_func=print_func, allow_newlines=allow_newlines, help_msg=help_msg)
     elif expected_type == FLOAT_TYPE:
         try:
             return convert_string_to_float(data)
         except ValueError:
             print_func("invalid entry. try again")
-            return get_user_input(message, expected_type, can_cancel, print_func)
+            return get_user_input(msg, expected_type, can_cancel=can_cancel, print_func=print_func, allow_newlines=allow_newlines, help_msg=help_msg)
     else:
         raise NotImplementedError
 
@@ -95,6 +106,13 @@ def convert_string_to_float(data: str):
 
 
 def set_term_title(name: str):
+    """
+    Description:
+        sets the terminal title cross system compatible
+    Parameters:
+        :param name: The title you name the terminal string
+        :return: None
+    """
     if PLTFRM in ("linux", "darwin"):
         sys.stdout.write(f"\x1b]2;{name}\x07")
     elif PLTFRM in ("win32", "cygwin"):
